from typing import Optional, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConstrainedSketchDimension import ConstrainedSketchDimension
from ..ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)
from ...UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class RadialDimension(ConstrainedSketchDimension):
    @abaqus_method_doc
    def __init__(
        self,
        curve: ConstrainedSketchGeometry,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
        majorRadius: Optional[float] = None,
        minorRadius: Optional[float] = None,
    ):
        """This method constructs a ConstrainedSketchDimension object on a circular or elliptical
        arc. A radial dimension indicates the radius of an arc or circle or the major or minor
        radius of an ellipse.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sketches[name].RadialDimension

        Parameters
        ----------
        curve
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object specifying the circular or elliptical arc.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float specifying the radius of the arc, circle or ellipse.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the angle between two lines.
        majorRadius
            A Float specifying the major Radius if **curve** is an ellipse. This is mutually exclusive
            with **value** and **minorRadius**.
        minorRadius
            A Float specifying the minor Radius if **curve** is an ellipse. This is mutually exclusive
            with **value** and **majorRadius**.

        Returns
        -------
        ConstrainedSketchDimension
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchDimension.ConstrainedSketchDimension.ConstrainedSketchDimension` object (None if the dimension cannot be created).

        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .._OptionsBase import _OptionsBase


@abaqus_class_doc
class AttributeColorMap(_OptionsBase):
    """The AttributeColorMap object is used to store values and attributes associated with
    AttributeColorMap type objects. AttributeColorMap objects can be modified using the
    methods described below. The methods accessed via the Viewport object cause the
    AttributeColorMap object to be updated in the session.viewports[name].colorMappings
    repository.

    .. note::
        This object can be accessed by::

            session.viewports[name].colorMappings[name]
    """

    #: A SymbolicConstant specifying the type of AttributeColorMap . Possible values are
    #: MATERIAL_MAP, SECTION_MAP, PART_MAP, ELSET_MAP, AVERAGING_REGION_MAP, and ELTYPE_MAP.
    mapType: Optional[SymbolicConstant] = None

    #: A Dictionary object specifying a color mapping. Each key is of String type and specifies
    #: an attribute in the map; the corresponding values specify the color definition to apply
    #: to that attribute in the form (0|1, wire color, edge color, face color). The 0|1 defines
    #: the active status for the attribute. For example:
    # `overrides = {'Part-1':(1,'#00FF00', '#00CCFF', '#00FF00')}`
    overrides: Optional[dict] = None

    #: A Dictionary object specifying a custom color mapping similar to overrides. For
    #: example: `defaultOverrides = {'Copper':(1,''#00FF00', '#00CCFF', '#00FF00')}`
    #: The color mapping can contain keys that have not been
    #: created. When the key is created, it gets the appropriate values from this mapping.
    defaultOverrides: Optional[dict] = None

    #: A Dictionary object specifying the color settings of each attribute as described in the
    #: :meth:`abaqus.Canvas.AttributeColorMap.updateOverrides` method.
    attributeColors: Optional[dict] = None

    @abaqus_method_doc
    def setDefaults(self):
        """This method resets the AttributeColorMap object to its default state."""
        #: TODO: Implement this method.
        ...

    @abaqus_method_doc
    def setValues(self, overrides: Optional[dict] = None, defaultOverrides: Optional[dict] = None):
        """This method modifies the AttributeColorMap object.

        Parameters
        ----------
        overrides
            A Dictionary object specifying a color mapping. Each key is of String type and specifies
            an attribute in the map; the corresponding values specify the color definition to apply
            to that attribute in the form (0|1, wire color, edge color, face color). The 0|1 defines
            the active status for the attribute. For example:
            `overrides = {'Part-1':(1,'#00FF00', '#00CCFF', '#00FF00')}`
        defaultOverrides
            A Dictionary object specifying a custom color mapping similar to overrides. For
            example: `defaultOverrides = {'Copper':(1,''#00FF00', '#00CCFF', '#00FF00')}`.
            The color mapping can contain keys that have not been
            created. When the key is created, it gets the appropriate values from this mapping.
        """
        super().setValues(overrides=overrides, defaultOverrides=defaultOverrides)

    @abaqus_method_doc
    def updateOverrides(self, overrides: Optional[dict] = None, defaultOverrides: Optional[dict] = None):
        """This method specifies additional overrides to be added to the current object definition.

        Parameters
        ----------
        overrides
            A Dictionary object specifying a color mapping. Each key is of String type and specifies
            an attribute in the map; the corresponding values specify the color definition to apply
            to that attribute in the form (0|1, wire color, edge color, face color). The 0|1 defines
            the active status for the attribute. For example:
            `overrides = {'Part-1':(1,'#00FF00', '#00CCFF', '#00FF00')}`
        defaultOverrides
            A Dictionary object specifying a custom color mapping similar to overrides. For
            example: `defaultOverrides={'Copper':(1,''#00FF00', '#00CCFF', '#00FF00')}`
            The color mapping can contain keys that have not been
            created. When the key is created, it gets the appropriate values from this mapping.
        """
        #: TODO: Implement this method.
        ...



from typing import Union, Optional, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import (AXIS_1, AXIS_2, AXIS_3, Boolean, FIRST_FRAME, OFF, ON,
                                              SymbolicConstant, TRANSLATE)


@abaqus_class_doc
class ViewCut:
    """The ViewCut object is used to store values and attributes associate with ViewCut type
    objects. ViewCut objects can be created using the methods described below. The methods
    accessed via the OdbDisplay object cause the ViewCut object to be added to the
    session.viewports[name].odbDisplay.viewCuts repository.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.viewports[name].layers[name].odbDisplay.viewCuts[name]
            session.viewports[name].odbDisplay.viewCuts[name]
    """

    #: A Float specifying the rotation angle of the cut defined with a **shape** set to PLANE.
    angle: Optional[float] = None

    #: A SymbolicConstant specifying the type of motion for the cut defined with a **shape** set
    #: to PLANE. Possible values are TRANSLATE and ROTATE. The default value is TRANSLATE.
    motion: SymbolicConstant = TRANSLATE

    #: A Float specifying the position of the cut defined with a **shape** set to PLANE.
    position: Optional[float] = None

    #: A Float specifying the radius of the cut defined with a **shape** set to CYLINDER or
    #: SPHERE.
    radius: Optional[float] = None

    #: A SymbolicConstant specifying the rotation axis for the cut defined with a **shape** set
    #: to PLANE. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is AXIS_2.
    rotationAxis: SymbolicConstant = AXIS_2

    #: A Float specifying the value of the cut defined with a **shape** set to ISOSURFACE.
    value: Optional[float] = None

    #: A Boolean specifying whether to display the model above the cut. The default value is
    #: OFF.
    showModelAboveCut: Boolean = OFF

    #: A Boolean specifying whether to display the model on the cut. The default value is ON.
    showModelOnCut: Boolean = ON

    #: A Boolean specifying whether to display the model below the cut. The default value is
    #: ON.
    showModelBelowCut: Boolean = ON

    #: A Boolean specifying whether to display the free body cut. The default value is OFF.
    showFreeBodyCut: Boolean = OFF

    #: A Boolean specifying whether the cut is displayed.
    active: Boolean = OFF

    #: A pair of Floats specifying the acceptable range for positioning the cut.
    cutRange: Sequence[float] = ()

    #: A String specifying the repository key.
    name: str

    #: A SymbolicConstant specifying the shape of the ViewCut object. Possible values are
    #: PLANE, CYLINDER, SPHERE, and ISOSURFACE.
    shape: SymbolicConstant

    #: A sequence of three Floats specifying the X-, Y-, and Z-coordinates of the origin of the
    #: plane, cylinder or sphere cut. This origin is not required if the cut shape is
    #: ISOSURFACE or if the cut is defined by the **csysName** argument.
    origin: tuple

    #: A sequence of Floats specifying the X-, Y-, and Z-coordinates of the normal axis to the
    #: plane defining the cut, when the plane is defined using the **origin** argument or a
    #: SymbolicConstant defining this normal axis, when the cut is defined by the **csysName**
    #: argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is not required if the
    #: cut **shape** is CYLINDER, SPHERE or ISOSURFACE.
    normal: Union[SymbolicConstant, float]

    #: A sequence of three Floats specifying the X-, Y-, and Z-coordinates of the second axis
    #: of the plane defining the cut, when the plane is defined using the **origin** argument or
    #: a SymbolicConstant defining this second axis, when the cut is defined by the **csysName**
    #: argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is used to rotate the
    #: plane cut. It is not required if the cut **shape** is CYLINDER, SPHERE or ISOSURFACE.
    axis2: Union[SymbolicConstant, float]

    #: A String specifying the name of the DatumCsys object to be used to define the cut. This
    #: name is not required if the cut **shape** is ISOSURFACE or if the cut is defined by the
    #: **origin** argument.
    csysName: str

    #: A sequence of Floats specifying the X-, Y-, and Z-coordinates of the cylinder axis
    #: defining the cut, when the cut is defined using the **origin** argument or a
    #: SymbolicConstant defining this cylinder axis, when the cut is defined by the **csysName**
    #: argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is not required if the
    #: cut **shape** is PLANE, SPHERE, or ISOSURFACE.
    cylinderAxis: Union[SymbolicConstant, float]

    #: A Boolean specifying whether the cut will follow the deformation or be static. The
    #: default value is OFF.
    followDeformation: Boolean = OFF

    #: A Boolean specifying averaging for element based fields associated with isosurface cuts
    #: will be set to compute-average with a threshold of 100% when true. The current field
    #: options will be used when false. The default value is ON.
    overrideAveraging: Boolean = ON

    #: A SymbolicConstant specifying which reference frame will be used when the cut follows
    #: the deformation. Possible values are FIRST_FRAME, LAST_FRAME, and CURRENT_FRAME. The
    #: default value is FIRST_FRAME.
    referenceFrame: SymbolicConstant = FIRST_FRAME

    #: A Float returning the cross-sectional area of the cut when **showFreeBodyCut** is set to
    #: ON.
    #:
    #: ..versionadded:: 2018
    #:     The `crossSectionalArea` attribute was added.
    crossSectionalArea: Optional[float] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        shape: SymbolicConstant,
        origin: tuple,
        normal: Union[SymbolicConstant, float],
        axis2: Union[SymbolicConstant, float],
        csysName: str,
        cylinderAxis: Union[SymbolicConstant, float],
        followDeformation: Boolean = OFF,
        overrideAveraging: Boolean = ON,
        referenceFrame: SymbolicConstant = FIRST_FRAME,
    ):
        """This method creates a ViewCut object.

        .. note:: 
            This function can be accessed by::

                session.viewports[name].layers[name].odbDisplay.ViewCut
                session.viewports[name].odbDisplay.ViewCut

        Parameters
        ----------
        name
            A String specifying the repository key.
        shape
            A SymbolicConstant specifying the shape of the ViewCut object. Possible values are
            PLANE, CYLINDER, SPHERE, and ISOSURFACE.
        origin
            A sequence of three Floats specifying the X-, Y-, and Z-coordinates of the origin of the
            plane, cylinder or sphere cut. This origin is not required if the cut shape is
            ISOSURFACE or if the cut is defined by the **csysName** argument.
        normal
            A sequence of Floats specifying the X-, Y-, and Z-coordinates of the normal axis to the
            plane defining the cut, when the plane is defined using the **origin** argument or a
            SymbolicConstant defining this normal axis, when the cut is defined by the **csysName**
            argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is not required if the
            cut **shape** is CYLINDER, SPHERE or ISOSURFACE.
        axis2
            A sequence of three Floats specifying the X-, Y-, and Z-coordinates of the second axis
            of the plane defining the cut, when the plane is defined using the **origin** argument or
            a SymbolicConstant defining this second axis, when the cut is defined by the **csysName**
            argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is used to rotate the
            plane cut. It is not required if the cut **shape** is CYLINDER, SPHERE or ISOSURFACE.
        csysName
            A String specifying the name of the DatumCsys object to be used to define the cut. This
            name is not required if the cut **shape** is ISOSURFACE or if the cut is defined by the
            **origin** argument.
        cylinderAxis
            A sequence of Floats specifying the X-, Y-, and Z-coordinates of the cylinder axis
            defining the cut, when the cut is defined using the **origin** argument or a
            SymbolicConstant defining this cylinder axis, when the cut is defined by the **csysName**
            argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is not required if the
            cut **shape** is PLANE, SPHERE, or ISOSURFACE.
        followDeformation
            A Boolean specifying whether the cut will follow the deformation or be static. The
            default value is OFF.
        overrideAveraging
            A Boolean specifying averaging for element based fields associated with isosurface cuts
            will be set to compute-average with a threshold of 100% when true. The current field
            options will be used when false. The default value is ON.
        referenceFrame
            A SymbolicConstant specifying which reference frame will be used when the cut follows
            the deformation. Possible values are FIRST_FRAME, LAST_FRAME, and CURRENT_FRAME. The
            default value is FIRST_FRAME.

        Returns
        -------
        ViewCut
            A :py:class:`~abaqus.OdbDisplay.ViewCut.ViewCut` object.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        angle: Optional[float] = None,
        motion: SymbolicConstant = TRANSLATE,
        position: Optional[float] = None,
        radius: Optional[float] = None,
        rotationAxis: SymbolicConstant = AXIS_2,
        value: Optional[float] = None,
        showModelAboveCut: Boolean = OFF,
        showModelOnCut: Boolean = ON,
        showModelBelowCut: Boolean = ON,
        showFreeBodyCut: Boolean = OFF,
        csysName: str = "",
        origin: tuple = (),
        normal: Union[SymbolicConstant, float] = AXIS_1,
        axis2: Union[SymbolicConstant, float] = AXIS_2,
        cylinderAxis: Union[SymbolicConstant, float] = AXIS_3,
        followDeformation: Boolean = OFF,
        overrideAveraging: Boolean = ON,
        referenceFrame: SymbolicConstant = FIRST_FRAME,
    ):
        """This method modifies the ViewCut object.

        Parameters
        ----------
        angle
            A Float specifying the rotation angle of the cut defined with a **shape** set to PLANE.
        motion
            A SymbolicConstant specifying the type of motion for the cut defined with a **shape** set
            to PLANE. Possible values are TRANSLATE and ROTATE. The default value is TRANSLATE.
        position
            A Float specifying the position of the cut defined with a **shape** set to PLANE.
        radius
            A Float specifying the radius of the cut defined with a **shape** set to CYLINDER or
            SPHERE.
        rotationAxis
            A SymbolicConstant specifying the rotation axis for the cut defined with a **shape** set
            to PLANE. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is AXIS_2.
        value
            A Float specifying the value of the cut defined with a **shape** set to ISOSURFACE.
        showModelAboveCut
            A Boolean specifying whether to display the model above the cut. The default value is
            OFF.
        showModelOnCut
            A Boolean specifying whether to display the model on the cut. The default value is ON.
        showModelBelowCut
            A Boolean specifying whether to display the model below the cut. The default value is
            ON.
        showFreeBodyCut
            A Boolean specifying whether to display the free body cut. The default value is OFF.
        csysName
            A String specifying the name of the DatumCsys object to be used to define the cut. This
            name is not required if the cut **shape** is ISOSURFACE or if the cut is defined by the
            **origin** argument.
        origin
            A sequence of three Floats specifying the X-, Y-, and Z-coordinates of the origin of the
            plane, cylinder or sphere cut. This origin is not required if the cut shape is
            ISOSURFACE or if the cut is defined by the **csysName** argument.
        normal
            A sequence of Floats specifying the X-, Y-, and Z-coordinates of the normal axis to the
            plane defining the cut, when the plane is defined using the **origin** argument or a
            SymbolicConstant defining this normal axis, when the cut is defined by the **csysName**
            argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is not required if the
            cut **shape** is CYLINDER, SPHERE or ISOSURFACE.
        axis2
            A sequence of three Floats specifying the X-, Y-, and Z-coordinates of the second axis
            of the plane defining the cut, when the plane is defined using the **origin** argument or
            a SymbolicConstant defining this second axis, when the cut is defined by the **csysName**
            argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is used to rotate the
            plane cut. It is not required if the cut **shape** is CYLINDER, SPHERE or ISOSURFACE.
        cylinderAxis
            A sequence of Floats specifying the X-, Y-, and Z-coordinates of the cylinder axis
            defining the cut, when the cut is defined using the **origin** argument or a
            SymbolicConstant defining this cylinder axis, when the cut is defined by the **csysName**
            argument. Possible values are AXIS_1, AXIS_2, AXIS_3. This axis is not required if the
            cut **shape** is PLANE, SPHERE, or ISOSURFACE.
        followDeformation
            A Boolean specifying whether the cut will follow the deformation or be static. The
            default value is OFF.
        overrideAveraging
            A Boolean specifying averaging for element based fields associated with isosurface cuts
            will be set to compute-average with a threshold of 100% when true. The current field
            options will be used when false. The default value is ON.
        referenceFrame
            A SymbolicConstant specifying which reference frame will be used when the cut follows
            the deformation. Possible values are FIRST_FRAME, LAST_FRAME, and CURRENT_FRAME. The
            default value is FIRST_FRAME.
        """
        ...

    @abaqus_method_doc
    def updateVariable(self):
        """This method updates the field associated with an isosurface cut to be consistent with
        the current primary variable.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from ..UtilityAndView.abaqusConstants import (BELOW_MIN, GLOBAL_X, MODEL, SEMI_AUTOMATIC,
                                              SymbolicConstant)


@abaqus_class_doc
class MassScaling:
    """A :py:class:`~abaqus.StepMiscellaneous.MassScaling.MassScaling` object defines the region and controls that govern mass scaling.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].massScaling[i]
    """

    #: A SymbolicConstant specifying the objective of the mass scaling definition. Possible
    #: values are SEMI_AUTOMATIC, AUTOMATIC, and REINITIALIZE. The default value is
    #: SEMI_AUTOMATIC.
    objective: SymbolicConstant = SEMI_AUTOMATIC

    #: A SymbolicConstant specifying whether mass scaling should be performed at the beginning
    #: of the step or throughout the step. Possible values are AT_BEGINNING and
    #: THROUGHOUT_STEP.
    occurs: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the type of scaling. Possible values are UNIFORM,
    #: BELOW_MIN, SET_EQUAL_DT, and ROLLING. The default value is BELOW_MIN.
    type: SymbolicConstant = BELOW_MIN

    #: A Float specifying a scaling factor.
    factor: Optional[float] = None

    #: A Float specifying a target time increment.
    dt: Optional[float] = None

    #: An Int specifying the frequency at which mass scaling calculations are performed.
    frequency: Optional[int] = None

    #: An Int specifying the number of intervals at which mass scaling calculations are
    #: performed.
    numberInterval: Optional[int] = None

    #: A Float specifying the estimated average velocity of the workpiece in the rolling
    #: direction at steady-state conditions.
    feedRate: Optional[float] = None

    #: A Float specifying the average element length in the extruded direction.
    extrudedLength: Optional[float] = None

    #: An Int specifying the number of nodes in the cross-section of the workpiece.
    crossSection: Optional[int] = None

    #: A SymbolicConstant specifying the rolling direction. Possible values are GLOBAL_X,
    #: GLOBAL_Y, GLOBAL_Z, and GLOBAL_NONE. The default value is GLOBAL_X.
    direction: SymbolicConstant = GLOBAL_X

    #: The SymbolicConstant MODEL or a Region object specifying where the mass scaling is
    #: applied. The default value is MODEL.
    region: SymbolicConstant = MODEL



from typing import Union, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Interaction import Interaction
from ..Datum.DatumAxis import DatumAxis
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (Boolean, COMPUTED, DEFAULT, KINEMATIC, OFF, OMIT,
                                              SymbolicConstant)


@abaqus_class_doc
class SurfaceToSurfaceContactExp(Interaction):
    """The SurfaceToSurfaceContactExp object defines surface-to-surface contact during an
    Abaqus/Explicit analysis.
    The SurfaceToSurfaceContactExp object is derived from the Interaction object.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name]
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the step in which the SurfaceToSurfaceContactExp object
    #: is created.
    createStepName: str

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the main surface.
    #:
    #: .. versionchanged:: 2022
    #:     The attribute `master` was renamed to `main`.
    main: Region

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the secondary surface.
    #:
    #: .. versionchanged:: 2022
    #:     The attribute `slave` was renamed to `secondary`.
    secondary: Region

    #: A SymbolicConstant specifying the contact formulation. Possible values are FINITE and
    #: SMALL.
    sliding: SymbolicConstant

    #: A String specifying the name of the ContactProperty object associated with this
    #: interaction.
    interactionProperty: str

    #: A SymbolicConstant specifying the mechanical constraint formulation. Possible values are
    #: KINEMATIC and PENALTY. The default value is KINEMATIC.
    mechanicalConstraint: SymbolicConstant = KINEMATIC

    #: A SymbolicConstant specifying the weighting for node-to-face contact. Possible values
    #: are DEFAULT and SPECIFIED. The default value is DEFAULT.
    weightingFactorType: SymbolicConstant = DEFAULT

    #: A Float specifying the weighting factor for the contact surfaces when
    #: **weightingFactorType** = SPECIFIED. The default value is 0.0.
    weightingFactor: float = 0

    #: A String specifying the name of the ContactControl object associated with this
    #: interaction. An empty string indicates that the default contact controls will be used.
    #: The default value is an empty string.
    contactControls: str = ""

    #: A SymbolicConstant or a Float specifying the initial clearance at regions of contact.
    #: Possible values are OMIT and COMPUTED. The default value is OMIT.
    initialClearance: Union[SymbolicConstant, float] = OMIT

    #: None or a sequence of Floats specifying the half thread angle used for bolt clearance.
    #: The default value is None.
    halfThreadAngle: Optional[str] = None

    #: None or a sequence of Floats specifying the pitch used for bolt clearance. The default
    #: value is None.
    pitch: Optional[str] = None

    #: The SymbolicConstant COMPUTED or a Float specifying the major diameter of the bolt used
    #: for bolt clearance. The default value is COMPUTED.
    majorBoltDiameter: Union[SymbolicConstant, float] = COMPUTED

    #: The SymbolicConstant COMPUTED or a Float specifying the mean diameter of the bolt used
    #: for bolt clearance. The default value is COMPUTED.
    meanBoltDiameter: Union[SymbolicConstant, float] = COMPUTED

    #: A :py:class:`~abaqus.Datum.DatumAxis.DatumAxis` object specifying the orientation of the bolt hole when specifying bolt
    #: clearance.
    datumAxis: Optional[DatumAxis] = None

    #: A Boolean specifying whether to reverse the bolt clearance direction given by the datum
    #: axis. The default value is OFF.
    useReverseDatumAxis: Boolean = OFF

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the contact region for which clearance is specified.
    clearanceRegion: Optional[Region] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        main: Region,
        secondary: Region,
        sliding: SymbolicConstant,
        interactionProperty: str,
        mechanicalConstraint: SymbolicConstant = KINEMATIC,
        weightingFactorType: SymbolicConstant = DEFAULT,
        weightingFactor: float = 0,
        contactControls: str = "",
        initialClearance: Union[SymbolicConstant, float] = OMIT,
        halfThreadAngle: Optional[str] = None,
        pitch: Optional[str] = None,
        majorBoltDiameter: Union[SymbolicConstant, float] = COMPUTED,
        meanBoltDiameter: Union[SymbolicConstant, float] = COMPUTED,
        datumAxis: Optional[DatumAxis] = None, 
        useReverseDatumAxis: Boolean = OFF,
        clearanceRegion: Optional[Region] = None,
    ):
        """This method creates a SurfaceToSurfaceContactExp object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].SurfaceToSurfaceContactExp

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the SurfaceToSurfaceContactExp object
            is created.
        main
            A :py:class:`~abaqus.Region.Region.Region` object specifying the main surface.

            .. versionchanged:: 2022
                The argument `master` was renamed to `main`.
        secondary
            A :py:class:`~abaqus.Region.Region.Region` object specifying the secondary surface.

            .. versionchanged:: 2022
                The argument `slave` was renamed to `secondary`.
        sliding
            A SymbolicConstant specifying the contact formulation. Possible values are FINITE and
            SMALL.
        interactionProperty
            A String specifying the name of the ContactProperty object associated with this
            interaction.
        mechanicalConstraint
            A SymbolicConstant specifying the mechanical constraint formulation. Possible values are
            KINEMATIC and PENALTY. The default value is KINEMATIC.
        weightingFactorType
            A SymbolicConstant specifying the weighting for node-to-face contact. Possible values
            are DEFAULT and SPECIFIED. The default value is DEFAULT.
        weightingFactor
            A Float specifying the weighting factor for the contact surfaces when
            **weightingFactorType** = SPECIFIED. The default value is 0.0.
        contactControls
            A String specifying the name of the ContactControl object associated with this
            interaction. An empty string indicates that the default contact controls will be used.
            The default value is an empty string.
        initialClearance
            A SymbolicConstant or a Float specifying the initial clearance at regions of contact.
            Possible values are OMIT and COMPUTED. The default value is OMIT.
        halfThreadAngle
            None or a sequence of Floats specifying the half thread angle used for bolt clearance.
            The default value is None.
        pitch
            None or a sequence of Floats specifying the pitch used for bolt clearance. The default
            value is None.
        majorBoltDiameter
            The SymbolicConstant COMPUTED or a Float specifying the major diameter of the bolt used
            for bolt clearance. The default value is COMPUTED.
        meanBoltDiameter
            The SymbolicConstant COMPUTED or a Float specifying the mean diameter of the bolt used
            for bolt clearance. The default value is COMPUTED.
        datumAxis
            A :py:class:`~abaqus.Datum.DatumAxis.DatumAxis` object specifying the orientation of the bolt hole when specifying bolt
            clearance.
        useReverseDatumAxis
            A Boolean specifying whether to reverse the bolt clearance direction given by the datum
            axis. The default value is OFF.
        clearanceRegion
            A :py:class:`~abaqus.Region.Region.Region` object specifying the contact region for which clearance is specified.

        Returns
        -------
        SurfaceToSurfaceContactExp
            A :py:class:`~abaqus.Interaction.SurfaceToSurfaceContactExp.SurfaceToSurfaceContactExp` object.
        """
        super().__init__()

    @abaqus_method_doc
    def swapSurfaces(self):
        """This method switches the main and secondary surfaces of a surface-to-surface contact
        pair. This command is valid only during the step in which the interaction is created.

        .. versionchanged:: 2022
            Master and slave were changed to main and secondary.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        mechanicalConstraint: SymbolicConstant = KINEMATIC,
        weightingFactorType: SymbolicConstant = DEFAULT,
        weightingFactor: float = 0,
        contactControls: str = "",
        initialClearance: Union[SymbolicConstant, float] = OMIT,
        halfThreadAngle: Optional[str] = None,
        pitch: Optional[str] = None,
        majorBoltDiameter: Union[SymbolicConstant, float] = COMPUTED,
        meanBoltDiameter: Union[SymbolicConstant, float] = COMPUTED,
        datumAxis: Optional[DatumAxis] = None, 
        useReverseDatumAxis: Boolean = OFF,
        clearanceRegion: Optional[Region] = None,
    ):
        """This method modifies the data for an existing SurfaceToSurfaceContactExp object in the
        step where it is created.

        Parameters
        ----------
        mechanicalConstraint
            A SymbolicConstant specifying the mechanical constraint formulation. Possible values are
            KINEMATIC and PENALTY. The default value is KINEMATIC.
        weightingFactorType
            A SymbolicConstant specifying the weighting for node-to-face contact. Possible values
            are DEFAULT and SPECIFIED. The default value is DEFAULT.
        weightingFactor
            A Float specifying the weighting factor for the contact surfaces when
            **weightingFactorType** = SPECIFIED. The default value is 0.0.
        contactControls
            A String specifying the name of the ContactControl object associated with this
            interaction. An empty string indicates that the default contact controls will be used.
            The default value is an empty string.
        initialClearance
            A SymbolicConstant or a Float specifying the initial clearance at regions of contact.
            Possible values are OMIT and COMPUTED. The default value is OMIT.
        halfThreadAngle
            None or a sequence of Floats specifying the half thread angle used for bolt clearance.
            The default value is None.
        pitch
            None or a sequence of Floats specifying the pitch used for bolt clearance. The default
            value is None.
        majorBoltDiameter
            The SymbolicConstant COMPUTED or a Float specifying the major diameter of the bolt used
            for bolt clearance. The default value is COMPUTED.
        meanBoltDiameter
            The SymbolicConstant COMPUTED or a Float specifying the mean diameter of the bolt used
            for bolt clearance. The default value is COMPUTED.
        datumAxis
            A :py:class:`~abaqus.Datum.DatumAxis.DatumAxis` object specifying the orientation of the bolt hole when specifying bolt
            clearance.
        useReverseDatumAxis
            A Boolean specifying whether to reverse the bolt clearance direction given by the datum
            axis. The default value is OFF.
        clearanceRegion
            A :py:class:`~abaqus.Region.Region.Region` object specifying the contact region for which clearance is specified.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self, stepName: str, interactionProperty: str = "", contactControls: str = ""
    ):
        """This method modifies the propagating data for an existing SurfaceToSurfaceContactExp
        object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the interaction is modified.
        interactionProperty
            A String specifying the name of the ContactProperty object associated with this
            interaction.
        contactControls
            A String specifying the name of the ContactControl object associated with this
            interaction. An empty string indicates that the default contact controls will be used.
            The default value is an empty string.
        """
        ...



from typing import Optional, Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Field import Field
from .FluidCavityPressure import FluidCavityPressure
from .InitialState import InitialState
from .KinematicHardening import KinematicHardening
from .MaterialAssignment import MaterialAssignment
from .PorePressure import PorePressure
from .Saturation import Saturation
from .Stress import Stress
from .Temperature import Temperature
from .Velocity import Velocity
from .VoidsRatio import VoidsRatio
from ..Assembly.PartInstanceArray import PartInstanceArray
from ..Model.ModelBase import ModelBase
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (Boolean, CONSTANT_THROUGH_THICKNESS, CONSTANT_RATIO,
                                              KINEMATIC_HARDENING, LAST_STEP, MAGNITUDE, OFF,
                                              STEP_END, SymbolicConstant, UNIFORM, UNSET)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class PredefinedFieldModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note:: 
        This object can be accessed by::

            mdb.models[name]
    """

    @abaqus_method_doc
    def FluidCavityPressure(
        self, name: str, fluidCavity: str, fluidPressure: float
    ) -> FluidCavityPressure:
        """This method creates a FluidCavityPressure object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].FluidCavityPressure

        Parameters
        ----------
        name
            A String specifying the repository key.
        fluidCavity
            A String specifying the name of a Fluid Cavity Interaction.
        fluidPressure
            A Float specifying the initial fluid pressure.

        Returns
        -------
        FluidCavityPressure
            A :py:class:`~abaqus.PredefinedField.FluidCavityPressure.FluidCavityPressure` object.
        """
        self.predefinedFields[name] = predefinedField = FluidCavityPressure(
            name, fluidCavity, fluidPressure
        )
        return predefinedField

    @abaqus_method_doc
    def InitialState(
        self,
        name: str,
        instances: PartInstanceArray,
        fileName: str,
        endStep: SymbolicConstant = LAST_STEP,
        endIncrement: SymbolicConstant = STEP_END,
        updateReferenceConfiguration: Boolean = OFF,
    ) -> InitialState:
        """This method creates an InitialState predefined field object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].InitialState

        Parameters
        ----------
        name
            A String specifying the repository key.
        instances
            A :py:class:`~abaqus.Assembly.PartInstanceArray.PartInstanceArray` object specifying the instances to which the predefined field is
            applied.
        fileName
            A String specifying the name of the job that generated the initial state data.
        endStep
            The SymbolicConstant LAST_STEP or an Int specifying the step from which the initial
            state values are to be read or the SymbolicConstant LAST_STEP. The default value is
            LAST_STEP.
        endIncrement
            The SymbolicConstant STEP_END or an Int specifying the increment, interval or iteration
            of the step set in **endStep** or the SymbolicConstant STEP_END. The default value is
            STEP_END.
        updateReferenceConfiguration
            A Boolean specifying whether to update the reference configuration based on the import
            data. The default value is OFF.

        Returns
        -------
        InitialState
            An :py:class:`~abaqus.PredefinedField.InitialState.InitialState` object.
        """
        self.predefinedFields[name] = predefinedField = InitialState(
            name,
            instances,
            fileName,
            endStep,
            endIncrement,
            updateReferenceConfiguration,
        )
        return predefinedField

    @abaqus_method_doc
    def KinematicHardening(
        self,
        name: str,
        region: Region,
        numBackStress: int = 1,
        equivPlasticStrain: tuple = (),
        backStress: tuple = (),
        sectPtNum: tuple = (),
        definition: SymbolicConstant = KINEMATIC_HARDENING,
        rebarLayerNames: tuple = (),
        distributionType: SymbolicConstant = MAGNITUDE,
    ) -> KinematicHardening:
        """This method creates a KinematicHardening object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].KinematicHardening

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the predefined field is applied.
        numBackStress
            An Int specifying the number of backstresses. The default value is 1.
        equivPlasticStrain
            A sequence of Floats specifying the initial equivalent Plastic strain.
        backStress
            A sequence of sequences of Floats specifying the initial backstress tensor for kinematic
            hardening models. The default value is an empty sequence.
        sectPtNum
            A sequence of Ints specifying section point numbers. This argument is valid only when
            **definition** = SECTION_PTS.
        definition
            A SymbolicConstant specifying different types of kinematic hardening. Possible values
            are KINEMATIC_HARDENING, CRUSHABLE_FOAM, REBAR, SECTION_PTS, and USER_DEFINED. The
            default value is KINEMATIC_HARDENING.
        rebarLayerNames
            A sequence of Strings specifying rebar layer names. This argument is valid only when
            **definition** = REBAR.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are MAGNITUDE
            and ANALYTICAL_FIELD. The default value is MAGNITUDE.

        Returns
        -------
        KinematicHardening
            A :py:class:`~abaqus.PredefinedField.KinematicHardening.KinematicHardening` object.
        """
        self.predefinedFields[name] = predefinedField = KinematicHardening(
            name,
            region,
            numBackStress,
            equivPlasticStrain,
            backStress,
            sectPtNum,
            definition,
            rebarLayerNames,
            distributionType,
        )
        return predefinedField

    @abaqus_method_doc
    def MaterialAssignment(
        self,
        name: str,
        instanceList: PartInstanceArray,
        useFields: Boolean = OFF,
        assignmentList: tuple = (),
        fieldList: tuple = (),
        colorList: tuple = (),
    ) -> MaterialAssignment:
        """This method creates a MaterialAssignment predefined field object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].MaterialAssignment

        Parameters
        ----------
        name
            A String specifying the repository key.
        instanceList
            A :py:class:`~abaqus.Assembly.PartInstanceArray.PartInstanceArray` object specifying the part instances to which the predefined field
            is applied. All instances must be assigned the same Eulerian section.
        useFields
            A Boolean specifying whether the volume fraction data will be uniform or defined by
            discrete fields. The default value is OFF.
        assignmentList
            A sequence of tuples specifying the uniform volume fractions to be assigned. This
            argument is valid only when **useFields** = FALSE. Each tuple contains two entries:A Region
            object.A tuple of Floats specifying the uniform volume fraction values. The length of
            the tuple must match the number of material instance names specified in the Eulerain
            section assigned to part instances specified by **instanceList**.
        fieldList
            A sequence of tuples specifying the discrete volume fractions to be assigned. This
            argument is valid only when **useFields** = TRUE. Each tuple contains two entries:A Region
            object.A tuple of Strings specifying Discrete Field names. The length of the tuple must
            match the number of material instance names specified in the Eulerain section assigned
            to part instances specified by **instanceList**.
        colorList
            A sequence of three Ints specifying colors used to display the material instance
            assignments. This is a sequence of R,G,B colors, where the values are represented by
            integers between 0 and 255. The default value is an empty sequence.

        Returns
        -------
        MaterialAssignment
            A :py:class:`~abaqus.PredefinedField.MaterialAssignment.MaterialAssignment` object.
        """
        self.predefinedFields[name] = predefinedField = MaterialAssignment(
            name, instanceList, useFields, assignmentList, fieldList, colorList
        )
        return predefinedField

    @abaqus_method_doc
    def PorePressure(
        self,
        name: str,
        region: Region,
        distributionType: Literal[C.UNIFORM, C.FROM_FILE, C.USER_DEFINED] = UNIFORM,
        porePressure1: float = ...,
        porePressure2: float = ...,
        coord1: float = ...,
        coord2: float = ...,
        pressure2Distribution: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
        pressure2Field: str = ...,
        variation: Literal[C.CONSTANT_RATIO, C.VARIABLE_RATIO] = CONSTANT_RATIO,
        fileName: str = ...,
        increment: Union[int, Literal[C.LAST_INCREMENT]] = ...,
        step: Union[int, Literal[C.LAST_STEP]] = ...,
        interpolate: Boolean = ...,
    ) -> PorePressure:
        """This method creates a PorePressure predefined field object.

        .. note::
        
            This function can be accessed by::

                mdb.models[name].PorePressure

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A Region object specifying the region to which the predefined field is applied. Region
            is ignored if the predefined field has **distributionType** = FROM_FILE.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            FROM_FILE and USER_DEFINED. The default value is UNIFORM.
        porePressure1
            The initial pore pressure in the first region in your model.
        porePressure2
            The pore pressure of the second location in your model
        coord1
            Vertical position of the first location in your model for which you are specifying initial pore pressure.
        coord2
            The vertical position of the second location in your model.
        pressure2Distribution
            A SymbolicConstant specifying either the magnitude of a uniform distribution for pore pressure at the
            second elevation or an analytical field to define a spatially varying initial pore pressure at the second elevation.
            Possible values are MAGNITUDE and ANALYTICAL_FIELD.
        pressure2Field
            A String specifying the name of the AnalyticalField object associated with this predefined field.
            The `pressure2Field` argument applies only when `distributionType` = ANALYTICAL_FIELD.
        variation
            A SymbolicConstant selecting the elevation distribution options, either Linear or Constant.
            Possible values are CONSTANT_RATIO and VARIABLE_RATIO.
        fileName
            A String specifying the name of the file from which the Field values are to be read when 
            `distributionType` = FROM_FILE.
        increment
            The SymbolicConstant LAST_INCREMENT or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        step
            The SymbolicConstant LAST_STEP or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        interpolate
            A Boolean specifying whether to interpolate a field read from an output database or results file.

        Returns
        -------
            A PorePressure object.
        """
        self.predefinedFields[name] = predefinedField = PorePressure(
            name,
            region,
            distributionType,
            porePressure1,
            porePressure2,
            coord1,
            coord2,
            pressure2Distribution,
            pressure2Field,
            variation,
            fileName,
            increment,
            step,
            interpolate,
        )
        return predefinedField

    @abaqus_method_doc
    def Temperature(
        self,
        name: str,
        createStepName: str,
        region: Region,
        distributionType: SymbolicConstant = UNIFORM,
        crossSectionDistribution: SymbolicConstant = CONSTANT_THROUGH_THICKNESS,
        field: str = "",
        amplitude: str = UNSET,
        fileName: str = "",
        beginStep: Optional[SymbolicConstant] = None,
        beginIncrement: Optional[SymbolicConstant] = None,
        endStep: Optional[SymbolicConstant] = None,
        endIncrement: Optional[SymbolicConstant] = None,
        interpolate: Union[SymbolicConstant, Boolean] = OFF,
        magnitudes: str = "",
        absoluteExteriorTolerance: float = 0,
        exteriorTolerance: float = 0,
    ) -> Temperature:
        """This method creates a Temperature object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Temperature

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the predefined field is created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the predefined field is applied. **Region**
            is ignored if the predefined field has a **distributionType** member available, and
            **distributionType** = FROM_FILE .
        distributionType
            A SymbolicConstant specifying how the predefined field varies spatially. Possible values
            are UNIFORM, USER_DEFINED, FROM_FILE, FIELD, FROM_FILE_AND_USER_DEFINED, and
            DISCRETE_FIELD. The default value is UNIFORM.
        crossSectionDistribution
            A SymbolicConstant specifying how the predefined field is distributed over the cross
            section of the region. Possible values are
            
            - CONSTANT_THROUGH_THICKNESS
            - GRADIENTS_THROUGH_SHELL_CS
            - GRADIENTS_THROUGH_BEAM_CS
            - POINTS_THROUGH_SECTION
            
            The default value is CONSTANT_THROUGH_THICKNESS.
        field
            A String specifying the name of the AnalyticalField or DiscreteField object associated
            with this predefined field. The **field** argument applies only when
            **distributionType** = FIELD or **distributionType** = DISCRETE_FIELD. The default value is an
            empty string.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the predefined field has no amplitude reference. The default
            value is UNSET.Note:*amplitude* should be given only if it is valid for the specified
            step.
        fileName
            A String specifying the name of the file from which the temperature values are to be
            read when **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED.
        beginStep
            An Int specifying the first step from which temperature values are to be read or the
            SymbolicConstant FIRST_STEP or LAST_STEP. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        beginIncrement
            An Int specifying the first increment of the step set in **beginStep** or the
            SymbolicConstants STEP_START or STEP_END. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        endStep
            An Int specifying the last step from which temperature values are to be read or the
            SymbolicConstants FIRST_STEP and LAST_STEP. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        endIncrement
            An Int specifying the last increment of the step set in **endStep** or the
            SymbolicConstants STEP_START and STEP_END. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        interpolate
            A SymbolicConstant specifying whether to interpolate a field read from an output
            database or results file. Possible values are OFF, ON or MIDSIDE_ONLY. The default value
            is OFF.
        magnitudes
            A Sequence of Doubles specifying the temperature values when **distributionType** = UNIFORM
            or FIELD. The value of the **magnitudes** argument is a function of the
            **crossSectionDistribution** argument, as shown in the following list:
            
            - If **crossSectionDistribution** = CONSTANT_THROUGH_THICKNESS then **magnitudes** is a Double
              specifying the temperature.
            - If **crossSectionDistribution** = GRADIENTS_THROUGH_SHELL_CS then **magnitudes** is a
              sequence of Doubles specifying the mean value and the gradient in the thickness
              direction.
            - If **crossSectionDistribution** = GRADIENTS_THROUGH_BEAM_CS then **magnitudes** is a
              sequence of Doubles specifying the mean value, the gradient in the N1 direction, and the
              gradient in the N2 direction.
            - If **crossSectionDistribution** = POINTS_THROUGH_SECTION then **magnitudes** is a sequence
              of Doubles specifying the temperature at each point.
        absoluteExteriorTolerance
            A Float specifying the absolute value by which a driven node of the field can lie
            outside the region of the elements of the global model. The default value is 0.0. This
            argument cannot be used with **midside**.
        exteriorTolerance
            A Float specifying the fraction of the average element size in the global model by which
            a driven node of the field can lie outside the region of the elements of the global
            model. The default value is 0.0. This argument cannot be used with **midside**.

        Returns
        -------
        Temperature
            A :py:class:`~abaqus.PredefinedField.Temperature.Temperature` object.
        """
        self.predefinedFields[name] = predefinedField = Temperature(
            name,
            createStepName,
            region,
            distributionType,
            crossSectionDistribution,
            field,
            amplitude,
            fileName,
            beginStep,
            beginIncrement,
            endStep,
            endIncrement,
            interpolate,
            magnitudes,
            absoluteExteriorTolerance,
            exteriorTolerance,
        )
        return predefinedField

    @abaqus_method_doc
    def Velocity(
        self,
        name: str,
        region: Region,
        velocity1: float,
        velocity2: float,
        velocity3: float,
        omega: float,
        axisBegin: tuple,
        axisEnd: tuple,
        field: str = "",
        distributionType: SymbolicConstant = MAGNITUDE,
    ) -> Velocity:
        """This method creates a Velocity predefined field object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Velocity

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the predefined field is applied.
        velocity1
            A Float specifying the first component of the velocity.
        velocity2
            A Float specifying the second component of the velocity.
        velocity3
            A Float specifying the third component of the velocity.
        omega
            A Float specifying the angular velocity.
        axisBegin
            A sequence of Floats specifying the *X-*, *Y-*, and **Z**- coordinates of the starting
            point of the axis about which **omega** is defined.
        axisEnd
            A sequence of Floats specifying the *X-*, *Y-*, and **Z**- coordinates of the end point of
            the axis about which **omega** is defined.
        field
            A String specifying the name of the AnalyticalField object associated with this
            predefined field. The **field** argument applies only when
            **distributionType** = FIELD_ANALYTICAL. The default value is an empty string.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are MAGNITUDE
            and FIELD_ANALYTICAL. The default value is MAGNITUDE.

        Returns
        -------
        Velocity
            A :py:class:`~abaqus.PredefinedField.Velocity.Velocity` object.
        """
        self.predefinedFields[name] = predefinedField = Velocity(
            name,
            region,
            velocity1,
            velocity2,
            velocity3,
            omega,
            axisBegin,
            axisEnd,
            field,
            distributionType,
        )
        return predefinedField

    @abaqus_method_doc
    def Saturation(
        self,
        name: str,
        region: Region,
        distributionType: Literal[C.UNIFORM, C.FIELD] = UNIFORM,
        field: str = '',
        value: float = ...
    ) -> Saturation:
        """This method creates a Saturation predefined field object.

        .. note::

            This function can be accessed by::

                mdb.models[name].Saturation

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A Region object specifying the region to which the predefined field is applied.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            and FIELD. The default value is UNIFORM.
        field
            A String specifying the name of the AnalyticalField object associated
            with this predefined field. The **field** argument applies only when
            **distributionType** = FIELD. The default value is an empty string.
        value
            A float specifying an initial saturation value for the region between 0.0 (for no
            saturation) and 1.0 (for full saturation).

        Returns
        -------
            A Saturation object.
        """
        self.predefinedFields[name] = predefinedField = Saturation(
            name,
            region,
            distributionType,
            field,
            value,
        )
        return predefinedField

    @abaqus_method_doc
    def Stress(
        self,
        name: str,
        region: Region,
        distributionType: Literal[C.UNIFORM, C.FROM_FILE] = UNIFORM,
        sigma11: float = ...,
        sigma22: float = ...,
        sigma33: float = ...,
        sigma12: float = ...,
        sigma13: float = ...,
        sigma23: float = ...,
    ) -> Stress:
        """This method creates a Stress predefined field object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Stress

        .. versionadded:: 2017
            The `Stress` method was added.

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A Region object specifying the region to which the predefined field is applied. Region
            is ignored if the predefined field has **distributionType** = FROM_FILE.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM
            and FROM_FILE. The default value is UNIFORM.
        sigma11
            A Float specifying the first principal component of the stress.
        sigma22
            A Float specifying the second principal component of the stress.
        sigma33
            A Float specifying the third principal component of the stress.
        sigma12
            A Float specifying the first shear component of the stress.
        sigma13
            A Float specifying the second shear component of the stress.
        sigma23
            A Float specifying the third shear component of the stress.

        Returns
        -------
            A Stress object.
        """
        self.predefinedFields[name] = predefinedField = Stress(
            name,
            region,
            distributionType,
            sigma11,
            sigma22,
            sigma33,
            sigma12,
            sigma13,
            sigma23,
        )
        return predefinedField
    
    @abaqus_method_doc
    def Field(
        self,
        name: str,
        createStepName: str,
        region: Region,
        outputVariable: str = "",
        fieldVariableNum: Optional[int] = None,
        distributionType: SymbolicConstant = UNIFORM,
        crossSectionDistribution: SymbolicConstant = CONSTANT_THROUGH_THICKNESS,
        field: str = "",
        amplitude: str = UNSET,
        fileName: str = "",
        beginStep: Optional[SymbolicConstant] = None,
        beginIncrement: Optional[SymbolicConstant] = None,
        endStep: Optional[SymbolicConstant] = None,
        endIncrement: Optional[SymbolicConstant] = None,
        interpolate: Union[SymbolicConstant, Boolean] = OFF,
        magnitudes: str = "",
    ):
        """This method creates a Field object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Field

        .. versionadded:: 2018
            The `Field` method was added.

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the predefined field is created.
        region
            A Region object specifying the region to which the predefined field is applied. *Region*
            is ignored if the predefined field has a **distributionType** member available, and
            **distributionType** = FROM_FILE.
        outputVariable
            A String specifying the scalar nodal output variable that will be read from an output
            database and used to initialize a specified predefined field. This argument is a
            required argument if **distributionType** = FROM_FILE or
            **distributionType** = FROM_FILE_AND_USER_DEFINED.
        fieldVariableNum
            An Int specifying the field variable number.
        distributionType
            A SymbolicConstant specifying how the predefined field varies spatially. Possible values
            are UNIFORM, USER_DEFINED, FROM_FILE, FIELD, FROM_FILE_AND_USER_DEFINED, and
            DISCRETE_FIELD. The default value is UNIFORM.
        crossSectionDistribution
            A SymbolicConstant specifying how the predefined field is distributed over the
            cross-section of the region. Possible values are
            
            - CONSTANT_THROUGH_THICKNESS
            - GRADIENTS_THROUGH_SHELL_CS
            - GRADIENTS_THROUGH_BEAM_CS
            - POINTS_THROUGH_SECTION
            
            The default value is CONSTANT_THROUGH_THICKNESS.
        field
            A String specifying the name of the AnalyticalField or DiscreteField object associated
            with this predefined field. The **field** argument applies only when
            **distributionType** = FIELD or **distributionType** = DISCRETE_FIELD. The default value is an
            empty string.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the predefined field has no amplitude reference. The default
            value is UNSET. 
            
            .. note::
                **amplitude** should be given only if it is valid for the specified step.
        fileName
            A String specifying the name of the file from which the Field values are to be read when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED.
        beginStep
            An Int specifying the first step from which Field values are to be read or the
            SymbolicConstant FIRST_STEP or LAST_STEP. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        beginIncrement
            An Int specifying the first increment of the step set in **beginStep** or the
            SymbolicConstants STEP_START or STEP_END. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        endStep
            An Int specifying the last step from which Field values are to be read or the
            SymbolicConstants FIRST_STEP and LAST_STEP. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        endIncrement
            An Int specifying the last increment of the step set in **endStep** or the
            SymbolicConstants STEP_START and STEP_END. This argument is valid only when
            **distributionType** = FROM_FILE or **distributionType** = FROM_FILE_AND_USER_DEFINED. The
            default value is None.
        interpolate
            A SymbolicConstant specifying whether to interpolate a field read from an output
            database or results file. Possible values are OFF, ON, or MIDSIDE_ONLY. The default
            value is OFF.
        magnitudes
            A Sequence of Doubles specifying the Field values when **distributionType** = UNIFORM or
            FIELD. The value of the **magnitudes** argument is a function of the
            **crossSectionDistribution** argument, as shown in the following list:
            
            - If **crossSectionDistribution** = CONSTANT_THROUGH_THICKNESS, **magnitudes** is a Double
              specifying the Field.
            - If **crossSectionDistribution** = GRADIENTS_THROUGH_SHELL_CS, **magnitudes** is a sequence
              of Doubles specifying the mean value and the gradient in the thickness direction.
            - If **crossSectionDistribution** = GRADIENTS_THROUGH_BEAM_CS, **magnitudes** is a sequence of
              Doubles specifying the mean value, the gradient in the N1 direction, and the gradient in
              the N2 direction.
            - If **crossSectionDistribution** = POINTS_THROUGH_SECTION, **magnitudes** is a sequence of
              Doubles specifying the Field at each point.

        Returns
        -------
            A Field object.
        """
        self.predefinedFields[name] = predefinedField = Field(
            name,
            createStepName,
            region,
            outputVariable,
            fieldVariableNum,
            distributionType,
            crossSectionDistribution,
            field,
            amplitude,
            fileName,
            beginStep,
            beginIncrement,
            endStep,
            endIncrement,
            interpolate,
            magnitudes,
        )
        return predefinedField

    @abaqus_method_doc
    def VoidsRatio(
        self,
        name: str,
        region: Region,
        distributionType: Literal[C.UNIFORM, C.FROM_FILE, C.USER_DEFINED] = UNIFORM,
        voidsRatio1: float = ...,
        voidsRatio2: float = ...,
        coord1: float = ...,
        coord2: float = ...,
        ratio2Distribution: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
        ratio2Field: str = ...,
        variation: Literal[C.CONSTANT_RATIO, C.VARIABLE_RATIO] = CONSTANT_RATIO,
        fileName: str = ...,
        increment: Union[int, Literal[C.LAST_INCREMENT]] = ...,
        step: Union[int, Literal[C.LAST_STEP]] = ...,
        interpolate: Boolean = ...,
    ) -> VoidsRatio:
        """This method creates a PorePressure predefined field object.

        .. note::
        This function can be accessed by::

            mdb.models[name].PorePressure

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A Region object specifying the region to which the predefined field is applied. Region
            is ignored if the predefined field has **distributionType** = FROM_FILE.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            FROM_FILE and USER_DEFINED. The default value is UNIFORM.
        voidsRatio1
            The initial void ratio in the first region in your model.
        voidsRatio2
            The void ratio of the second location in your model
        coord1
            Vertical position of the first location in your model for which you are specifying initial void ratio.
        coord2
            The vertical position of the second location in your model
        ratio2Distribution
            A SymbolicConstant specifying either the magnitude of a uniform distribution for void ratio at the
            second elevation or an analytical field to define a spatially varying initial void ratio at the second elevation.
            Possible values are MAGNITUDE and ANALYTICAL_FIELD.
        ratio2Field
            A String specifying the name of the AnalyticalField object associated with this predefined field.
            The `ratio2Field` argument applies only when `distributionType` = ANALYTICAL_FIELD.
        variation
            A SymbolicConstant selecting the elevation distribution options, either Linear or Constant.
            Possible values are CONSTANT_RATIO and VARIABLE_RATIO.
        fileName
            A String specifying the name of the file from which the Field values are to be read when 
            `distributionType` = FROM_FILE.
        increment
            The SymbolicConstant LAST_INCREMENT or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        step
            The SymbolicConstant LAST_STEP or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        interpolate
            A Boolean specifying whether to interpolate a field read from an output database or results file.

        Returns
        -------
            A VoidsRatio object.
        """
        self.predefinedFields[name] = predefinedField = VoidsRatio(
            name,
            region,
            distributionType,
            voidsRatio1,
            voidsRatio2,
            coord1,
            coord2,
            ratio2Distribution,
            ratio2Field,
            variation,
            fileName,
            increment,
            step,
            interpolate,
        )
        return predefinedField



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class MainSecondaryAssignment:
    """The MainSecondaryAssignment object stores the main-secondary assignment definition for
    surfaces in ContactExp and ContactStd objects. The MainSecondaryAssignment object has no
    constructor or members.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name].mainSecondaryAssignments

        The corresponding analysis keywords are:

        - CONTACT FORMULATION

    .. versionchanged:: 2022
        The MasterSlaveAssignment class was renamed to MainSecondaryAssignment.
    """

    @abaqus_method_doc
    def changeValuesInStep(self, stepName: str, index: int, value: SymbolicConstant):
        """This method allows modification of main-secondary assignments already defined on surface
        pairs in a given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the main-secondary assignments are to
            be modified.
        index
            An Int specifying the position of the main-secondary assignment whose value is to be
            modified.
        value
            A SymbolicConstant specifying the value of the main-secondary role to be assigned to the
            surface whose index is referenced. Possible values are MAIN, SECONDARY, and BALANCED.
            The SymbolicConstant BALANCED can be specified only in an Abaqus/Standard analysis.
        """
        ...

    @abaqus_method_doc
    def appendInStep(self, stepName: str, assignments: SymbolicConstant):
        """This method allows addition of main-secondary assignments to new surface pairs in a
        given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the main-secondary assignments are to
            be defined.
        assignments
            A sequence of tuples specifying the main-secondary assignments. Each tuple contains two
            entries:
            
            - A region object or the SymbolicConstant GLOBAL specifying the surface to which the
              main-secondary attribute is assigned.
            - A SymbolicConstant specifying the overriding main-secondary value to be used for the
              first surface. Possible values of the SymbolicConstant are MAIN, SECONDARY, and
              BALANCED. The SymbolicConstant BALANCED can be specified only in an Abaqus/Standard
              analysis.
        """
        ...

    @abaqus_method_doc
    def delete(self, indices: tuple):
        """The delete method allows you to delete existing main-secondary assignments.

        Parameters
        ----------
        indices
            A sequence of Ints specifying the index of each main-secondary assignment to delete.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Interaction import Interaction
from ..UtilityAndView.abaqusConstants import Boolean, ON


@abaqus_class_doc
class XFEMCrackGrowth(Interaction):
    """The XFEMCrackGrowth object defines the enrichment activation state for an XFEMCrack.
    The XFEMCrackGrowth object is derived from the Interaction object.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name]

        The corresponding analysis keywords are:

        - ENRICHMENT ACTIVATION
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the step in which the XFEMCrackGrowth object is created.
    createStepName: str

    #: A String specifying the XFEMCrack object associated with this interaction.
    crackName: str

    #: A Boolean specifying whether the crack is allowed to grow (propagate) during this
    #: analysis step. The default value is ON.
    allowGrowth: Boolean = ON

    @abaqus_method_doc
    def __init__(
        self, name: str, createStepName: str, crackName: str, allowGrowth: Boolean = ON
    ):
        """This method creates an XFEMCrackGrowth object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].XFEMCrackGrowth

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the XFEMCrackGrowth object is created.
        crackName
            A String specifying the XFEMCrack object associated with this interaction.
        allowGrowth
            A Boolean specifying whether the crack is allowed to grow (propagate) during this
            analysis step. The default value is ON.

        Returns
        -------
        XFEMCrackGrowth
            A :py:class:`~abaqus.Interaction.XFEMCrackGrowth.XFEMCrackGrowth` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, allowGrowth: Boolean = ON):
        """This method modifies the data for an existing XFEMCrackGrowth object in the step where
        it is created.

        Parameters
        ----------
        allowGrowth
            A Boolean specifying whether the crack is allowed to grow (propagate) during this
            analysis step. The default value is ON.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str, allowGrowth: Boolean = ON):
        """This method modifies the propagating data for an existing XFEMCrackGrowth object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the interaction is modified.
        allowGrowth
            A Boolean specifying whether the crack is allowed to grow (propagate) during this
            analysis step. The default value is ON.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ....UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class FailureRatios:
    """The FailureRatios object specifies the shape of the failure surface for a Concrete
    model.

    .. note:: 
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].concrete.failureRatios
            import odbMaterial
            session.odbs[name].materials[name].concrete.failureRatios

        The table data for this object are:

        - Ratio of the ultimate biaxial compressive stress to the uniaxial compressive ultimate
          stress. The default value is 1.16.
        - Absolute value of the ratio of the uniaxial tensile stress at failure to the uniaxial 
          compressive stress at failure. The default value is 0.09.
        - Ratio of the magnitude of a principal component of Plastic strain at ultimate stress in 
          biaxial compression to the Plastic strain at ultimate stress in uniaxial compression. 
          The default value is 1.28.
        - Ratio of the tensile principal stress value at shear in plane stress, when the other 
          nonzero principal stress component is at the ultimate compressive stress value, to the 
          tensile cracking stress under uniaxial tension. The default value is 1/3.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - FAILURE RATIOS
    """

    @abaqus_method_doc
    def __init__(
        self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0
    ):
        """This method creates a FailureRatios object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].materials[name].concrete.FailureRatios
                session.odbs[name].materials[name].concrete.FailureRatios

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        FailureRatios
            A :py:class:`~abaqus.Material.Plastic.Concrete.FailureRatios.FailureRatios` object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the FailureRatios object.

        Raises
        ------
        RangeError
        """
        ...



from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .PredefinedField import PredefinedField
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, UNIFORM, CONSTANT_RATIO, MAGNITUDE
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class PorePressure(PredefinedField):
    """The PorePressure object stores the data for an initial pore pressure predefined field.
    The PorePressure object is derived from the PredefinedField object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].predefinedFields[name]

        The corresponding analysis keywords are:

        - INITIAL CONDITIONS
    """

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        region: Region,
        distributionType: Literal[C.UNIFORM, C.FROM_FILE, C.USER_DEFINED] = UNIFORM,
        porePressure1: float = ...,
        porePressure2: float = ...,
        coord1: float = ...,
        coord2: float = ...,
        pressure2Distribution: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
        pressure2Field: str = ...,
        variation: Literal[C.CONSTANT_RATIO, C.VARIABLE_RATIO] = CONSTANT_RATIO,
        fileName: str = ...,
        increment: Union[int, Literal[C.LAST_INCREMENT]] = ...,
        step: Union[int, Literal[C.LAST_STEP]] = ...,
        interpolate: Boolean = ...,
    ) -> None:
        """This method creates a PorePressure predefined field object.

        .. note::

            This function can be accessed by::

                mdb.models[name].PorePressure

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A Region object specifying the region to which the predefined field is applied. Region
            is ignored if the predefined field has **distributionType** = FROM_FILE.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            FROM_FILE and USER_DEFINED. The default value is UNIFORM.
        porePressure1
            The initial pore pressure in the first region in your model.
        porePressure2
            The pore pressure of the second location in your model
        coord1
            Vertical position of the first location in your model for which you are specifying initial pore pressure.
        coord2
            The vertical position of the second location in your model.
        pressure2Distribution
            A SymbolicConstant specifying either the magnitude of a uniform distribution for pore pressure at the
            second elevation or an analytical field to define a spatially varying initial pore pressure at the second elevation.
            Possible values are MAGNITUDE and ANALYTICAL_FIELD.
        pressure2Field
            A String specifying the name of the AnalyticalField object associated with this predefined field.
            The `pressure2Field` argument applies only when `distributionType` = ANALYTICAL_FIELD.
        variation
            A SymbolicConstant selecting the elevation distribution options, either Linear or Constant.
            Possible values are CONSTANT_RATIO and VARIABLE_RATIO.
        fileName
            A String specifying the name of the file from which the Field values are to be read when 
            `distributionType` = FROM_FILE.
        increment
            The SymbolicConstant LAST_INCREMENT or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        step
            The SymbolicConstant LAST_STEP or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        interpolate
            A Boolean specifying whether to interpolate a field read from an output database or results file.

        Returns
        -------
            A PorePressure object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        distributionType: Literal[C.UNIFORM, C.FROM_FILE, C.USER_DEFINED] = UNIFORM,
        porePressure1: float = ...,
        porePressure2: float = ...,
        coord1: float = ...,
        coord2: float = ...,
        pressure2Distribution: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
        pressure2Field: str = ...,
        variation: Literal[C.CONSTANT_RATIO, C.VARIABLE_RATIO] = CONSTANT_RATIO,
        fileName: str = ...,
        increment: Union[int, Literal[C.LAST_INCREMENT]] = ...,
        step: Union[int, Literal[C.LAST_STEP]] = ...,
        interpolate: bool = ...,
    ) -> None:
        """This method modifies the PorePressure object.

        Parameters
        ----------
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are UNIFORM,
            FROM_FILE and USER_DEFINED. The default value is UNIFORM.
        porePressure1
            The initial pore pressure in the first region in your model.
        porePressure2
            The pore pressure of the second location in your model
        coord1
            Vertical position of the first location in your model for which you are specifying initial pore pressure.
        coord2
            The vertical position of the second location in your model.
        pressure2Distribution
            A SymbolicConstant specifying either the magnitude of a uniform distribution for pore pressure at the
            second elevation or an analytical field to define a spatially varying initial pore pressure at the second elevation.
            Possible values are MAGNITUDE and ANALYTICAL_FIELD.
        pressure2Field
            A String specifying the name of the AnalyticalField object associated with this predefined field.
            The `pressure2Field` argument applies only when `distributionType` = ANALYTICAL_FIELD.
        variation
            A SymbolicConstant selecting the elevation distribution options, either Linear or Constant.
            Possible values are CONSTANT_RATIO and VARIABLE_RATIO.
        fileName
            A String specifying the name of the file from which the Field values are to be read when 
            `distributionType` = FROM_FILE.
        increment
            The SymbolicConstant LAST_INCREMENT or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        step
            The SymbolicConstant LAST_STEP or an Int specifying the increment, interval or iteration
            of the step when `distributionType` = FROM_FILE.
        interpolate
            A Boolean specifying whether to interpolate a field read from an output database or results file.

        Returns
        -------
            None
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Leaf import Leaf
from ..UtilityAndView.abaqusConstants import Boolean, OFF, SymbolicConstant


@abaqus_class_doc
class DisplayGroup:
    """DisplayGroup objects are used to select a subset of the entities displayed in the
    viewport.

    .. note:: 
        This object can be accessed by::

            session.displayGroups[name]
            import assembly
            session.viewports[name].assemblyDisplay.displayGroup
            session.viewports[name].layers[name].assemblyDisplay.displayGroup
            import visualization
            session.viewports[name].layers[name].odbDisplay.displayGroup
            import part
            session.viewports[name].layers[name].partDisplay.displayGroup
            session.viewports[name].odbDisplay.displayGroup
            session.viewports[name].partDisplay.displayGroup
    """

    #: A Boolean specifying whether Undo is possible or not.
    canUndo: Boolean = OFF

    #: A Boolean specifying whether Redo is possible or not.
    canRedo: Boolean = OFF

    #: A String specifying the repository key.
    name: str = ""

    #: A SymbolicConstant specifying the module in which the display group has been created.
    #: The possible values are PART, ASSEMBLY, PART_ASSEMBLY, ODB, and ALL.
    module: Optional[SymbolicConstant] = None

    #: A String specifying the name of the model to which the display group belongs when the
    #: module is part- or assembly-based.
    modelName: str = ""

    #: A String specifying the name of the part to which the display group belongs when the
    #: module is part-based.
    partName: str = ""

    @abaqus_method_doc
    def __init__(self, name: str, leaf: Leaf):
        """This method creates a DisplayGroup object.

        .. note:: 
            This function can be accessed by::

                session.DisplayGroup

        Parameters
        ----------
        name
            A String specifying the repository key.
        leaf
            A :py:class:`~abaqus.DisplayGroup.Leaf.Leaf` object specifying the items in the display group.

        Returns
        -------
        DisplayGroup
            A :py:class:`~abaqus.DisplayGroup.DisplayGroup.DisplayGroup` object.
        """
        ...

    @abaqus_method_doc
    def add(self, leaf: Leaf):
        """This method adds the specified items to the display group.

        Parameters
        ----------
        leaf
            A :py:class:`~abaqus.DisplayGroup.Leaf.Leaf` object specifying the items to add to the display group.
        """
        ...

    @abaqus_method_doc
    def either(self, leaf: Leaf):
        """This method redefines the display group to be only those items that are not shared by
        the **leaf** argument and by the display group.

        Parameters
        ----------
        leaf
            A :py:class:`~abaqus.DisplayGroup.Leaf.Leaf` object specifying the items to be excluded from the display group.
        """
        ...

    @abaqus_method_doc
    def intersect(self, leaf: Leaf):
        """This method redefines the display group to be only those items that are shared by the
        **leaf** argument and the display group.

        Parameters
        ----------
        leaf
            A :py:class:`~abaqus.DisplayGroup.Leaf.Leaf` object specifying the items to be included in the display group.
        """
        ...

    @abaqus_method_doc
    def redoLast(self):
        """This method redoes the last undone operation on the display group."""
        ...

    @abaqus_method_doc
    def remove(self, leaf: Leaf):
        """This method removes the specified items from the display group.

        Parameters
        ----------
        leaf
            A :py:class:`~abaqus.DisplayGroup.Leaf.Leaf` object specifying the items to remove from the display group.
        """
        ...

    @abaqus_method_doc
    def replace(self, leaf: Leaf):
        """This method replaces the contents of the display group with the specified items.

        Parameters
        ----------
        leaf
            A :py:class:`~abaqus.DisplayGroup.Leaf.Leaf` object specifying the items with which to replace the current display group
            contents.
        """
        ...

    @abaqus_method_doc
    def undoLast(self):
        """This method undoes the last operation performed on the display group."""
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class OdbMeshNode:
    """OdbMeshNode objects are created with the part.addNodes method.

    .. note:: 
        This object can be accessed by::

            import odbAccess
            session.odbs[name].parts[name].nodes[i]
            session.odbs[name].parts[name].nodeSets[name].nodes[i]
            session.odbs[name].parts[name].surfaces[name].nodes[i]
            session.odbs[name].rootAssembly.instances[name].nodes[i]
            session.odbs[name].rootAssembly.instances[name].nodeSets[name].nodes[i]
            session.odbs[name].rootAssembly.instances[name].surfaces[name].nodes[i]
            session.odbs[name].rootAssembly.nodes[i]
            session.odbs[name].rootAssembly.nodeSets[name].nodes[i]
            session.odbs[name].rootAssembly.surfaces[name].nodes[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.nodes[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.nodeSets[name].nodes[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.surfaces[name].nodes[i]
    """

    #: An Int specifying the node label.
    label: Optional[int] = None

    #: A tuple of Floats specifying the nodal coordinates in the global Cartesian coordinate
    #: system.
    coordinates: Optional[float] = None



from typing import Dict, Optional

from abqpy.decorators import abaqus_class_doc
from .Step import Step
from ..Adaptivity.AdaptiveMeshConstraintState import AdaptiveMeshConstraintState
from ..Adaptivity.AdaptiveMeshDomain import AdaptiveMeshDomain
from ..BoundaryCondition.BoundaryConditionState import BoundaryConditionState
from ..Load.LoadCase import LoadCase
from ..Load.LoadState import LoadState
from ..PredefinedField.PredefinedFieldState import PredefinedFieldState
from ..StepMiscellaneous.Control import Control
from ..StepMiscellaneous.SolverControl import SolverControl
from ..StepOutput.DiagnosticPrint import DiagnosticPrint
from ..StepOutput.FieldOutputRequestState import FieldOutputRequestState
from ..StepOutput.HistoryOutputRequestState import HistoryOutputRequestState
from ..StepOutput.Monitor import Monitor
from ..StepOutput.Restart import Restart
from ..UtilityAndView.abaqusConstants import Boolean, OFF, SymbolicConstant


@abaqus_class_doc
class AnalysisStep(Step):
    """The AnalysisStep object is the abstract base type for other Step objects, except the
    InitialStep object. It has no explicit constructor, no methods, and has only inherited
    members.
    The AnalysisStep object is derived from the Step object.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].steps[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A SymbolicConstant specifying whether the step has an explicit procedure type
    #: (*procedureType* = ANNEAL, DYNAMIC_EXPLICIT, or DYNAMIC_TEMP_DISPLACEMENT).
    explicit: Optional[SymbolicConstant] = None

    #: A Boolean specifying whether the step has a perturbation procedure type.
    perturbation: Boolean = OFF

    #: A Boolean specifying whether the step has a mechanical procedure type.
    nonmechanical: Boolean = OFF

    #: A SymbolicConstant specifying the Abaqus procedure. Possible values are:
    #: 
    #: - ANNEAL
    #: - BUCKLE
    #: - COMPLEX_FREQUENCY
    #: - COUPLED_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRIC
    #: - DIRECT_CYCLIC
    #: - DYNAMIC_IMPLICIT
    #: - DYNAMIC_EXPLICIT
    #: - DYNAMIC_SUBSPACE
    #: - DYNAMIC_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRICAL_STRUCTURAL
    #: - FREQUENCY
    #: - GEOSTATIC
    #: - HEAT_TRANSFER
    #: - MASS_DIFFUSION
    #: - MODAL_DYNAMICS
    #: - RANDOM_RESPONSE
    #: - RESPONSE_SPECTRUM
    #: - SOILS
    #: - STATIC_GENERAL
    #: - STATIC_LINEAR_PERTURBATION
    #: - STATIC_RIKS
    #: - STEADY_STATE_DIRECT
    #: - STEADY_STATE_MODAL
    #: - STEADY_STATE_SUBSPACE
    #: - VISCO
    procedureType: Optional[SymbolicConstant] = None

    #: A Boolean specifying whether the step is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    #: A repository of FieldOutputRequestState objects.
    fieldOutputRequestState: Dict[str, FieldOutputRequestState] = {}

    #: A repository of HistoryOutputRequestState objects.
    historyOutputRequestState: Dict[str, HistoryOutputRequestState] = {}

    #: A :py:class:`~abaqus.StepOutput.DiagnosticPrint.DiagnosticPrint` object.
    diagnosticPrint: DiagnosticPrint = DiagnosticPrint()

    #: A :py:class:`~abaqus.StepOutput.Monitor.Monitor` object.
    monitor: Optional[Monitor] = None

    #: A :py:class:`~abaqus.StepOutput.Restart.Restart` object.
    restart: Restart = Restart()

    #: A repository of AdaptiveMeshConstraintState objects.
    adaptiveMeshConstraintStates: Dict[str, AdaptiveMeshConstraintState] = {}

    #: A repository of AdaptiveMeshDomain objects.
    adaptiveMeshDomains: Dict[str, AdaptiveMeshDomain] = {}

    #: A :py:class:`~abaqus.StepMiscellaneous.Control.Control` object.
    control: Control = Control()

    #: A :py:class:`~abaqus.StepMiscellaneous.SolverControl.SolverControl` object.
    solverControl: SolverControl = SolverControl()

    #: A repository of BoundaryConditionState objects.
    boundaryConditionStates: Dict[str, BoundaryConditionState] = {}

    #: A repository of InteractionState objects.
    interactionStates: Optional[int] = None

    #: A repository of LoadState objects.
    loadStates: Dict[str, LoadState] = {}

    #: A repository of LoadCase objects.
    loadCases: Dict[str, LoadCase] = {}

    #: A repository of PredefinedFieldState objects.
    predefinedFieldStates: Dict[str, PredefinedFieldState] = {}



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .DataTableArray import DataTableArray
from .Field import Field
from ..Assembly.PartInstance import PartInstance
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (Boolean, CARTESIAN, INSIDE, MEDIUM, NODES, OFF,
                                              SymbolicConstant)


@abaqus_class_doc
class DiscreteField(Field):
    """The DiscreteField object defines a varying field whose values correspond to distinct
    points within a domain.
    The DiscreteField object is derived from the Field object.

    .. note:: 
        This object can be accessed by::

            import fields
            mdb.models[name].discreteFields[name]
    """

    #: A String specifying the repository key.
    name: str

    #: A sequence of Floats specifying a sequence of floats specifying the default values.
    defaultValues: tuple

    #: A SymbolicConstant or an Int specifying the type of data represented by this discrete
    #: field. Possible values are SCALAR, ORIENTATION, and PRESCRIBEDCONDITION_DOF.
    fieldType: SymbolicConstant

    #: A SymbolicConstant or an Int specifying the location of the domain data. Possible values
    #: are NODES and ELEMENTS. The default value is NODES.
    location: SymbolicConstant = NODES

    #: An Int specifying the width of the supplied data. The default value is 1.
    dataWidth: int = 1

    #: A :py:class:`~abaqus.Field.DataTableArray.DataTableArray` object.
    data: Optional[DataTableArray] = None

    #: A String specifying the description of the field. The default value is an empty string.
    description: str = ""

    #: A SymbolicConstant specifying the type of the system being described by a discrete field
    #: used for an orientation. Possible values are CARTESIAN, CYLINDRICAL, and SPHERICAL. The
    #: default value is CARTESIAN.
    orientationType: SymbolicConstant = CARTESIAN

    #: A Boolean specifying whether or not the orientations are described in terms of part
    #: level coordinates. The default value is OFF.
    partLevelOrientation: Boolean = OFF

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        defaultValues: tuple,
        fieldType: SymbolicConstant,
        location: SymbolicConstant = NODES,
        dataWidth: int = 1,
        data: Optional[DataTableArray] = None,
        description: str = "",
        orientationType: SymbolicConstant = CARTESIAN,
        partLevelOrientation: Boolean = OFF,
    ):
        """This method creates a DiscreteField object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].DiscreteField

        Parameters
        ----------
        name
            A String specifying the repository key.
        defaultValues
            A sequence of Floats specifying a sequence of floats specifying the default values.
        fieldType
            A SymbolicConstant or an Int specifying the type of data represented by this discrete
            field. Possible values are SCALAR, ORIENTATION, and PRESCRIBEDCONDITION_DOF.
        location
            A SymbolicConstant or an Int specifying the location of the domain data. Possible values
            are NODES and ELEMENTS. The default value is NODES.
        dataWidth
            An Int specifying the width of the supplied data. The default value is 1.
        data
            A :py:class:`~abaqus.Field.DataTableArray.DataTableArray` object.
        description
            A String specifying the description of the field. The default value is an empty string.
        orientationType
            A SymbolicConstant specifying the type of the system being described by a discrete field
            used for an orientation. Possible values are CARTESIAN, CYLINDRICAL, and SPHERICAL. The
            default value is CARTESIAN.
        partLevelOrientation
            A Boolean specifying whether or not the orientations are described in terms of part
            level coordinates. The default value is OFF.

        Returns
        -------
        DiscreteField
            A :py:class:`~abaqus.Field.DiscreteField.DiscreteField` object.

        Raises
        ------
        AbaqusException
        """
        super().__init__()

    @abaqus_method_doc
    def DiscreteFieldByVolumeFraction(
        self,
        name: str,
        eulerianInstance: PartInstance,
        referenceInstance: PartInstance,
        accuracy: str = MEDIUM,
        materialLocation: str = INSIDE,
        description: str = "",
        scaleFactor: str = "",
    ):
        """This method creates a DiscreteField object that represents the volume fraction of each
        element of an Eulerian Instance that is occupied by a reference instance.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].DiscreteField

        Parameters
        ----------
        name
            A String specifying the repository key.
        eulerianInstance
            A :py:class:`~abaqus.Assembly.PartInstance.PartInstance` object specifying the elements for which volume fraction values will be
            computed.
        referenceInstance
            A :py:class:`~abaqus.Assembly.PartInstance.PartInstance` object specifying the region that either contains material or is empty of
            material.
        accuracy
            A Symbolic Constant specifying the level of accuracy that will be used in computing
            volume fractions. Possible values are LOW, MEDIUM, or HIGH. The default value is MEDIUM.
        materialLocation
            A Symbolic Constant indicating whether the material is inside or outside the
            **referenceInstance**. Possible values are INSIDE or OUTSIDE. The default value is INSIDE.
        description
            A String specifying the description of the field. The default value is an empty string.
        scaleFactor
            A float specifying the fraction of the volume that is occupied by the
            *referenceInstance.* Valid values are between 0 and 1.

        Returns
        -------
        DiscreteField
            A :py:class:`~abaqus.Field.DiscreteField.DiscreteField` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DiscreteFieldFromAnalytic(
        self,
        name: str,
        location: SymbolicConstant,
        analyticFieldName: str,
        region: Region,
    ):
        """This method creates a DiscreteField object from a AnalyticalField object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].DiscreteField

        Parameters
        ----------
        name
            A String specifying the repository key.
        location
            A SymbolicConstant or an Int specifying the location of the domain data. Possible values
            are NODES and ELEMENTS. The default value is NODES.
        analyticFieldName
            A String specifying the name of the AnalyticalField containing the source data.
        region
            A :py:class:`~abaqus.Region.Region.Region` object for the field.

        Returns
        -------
        DiscreteField
            A :py:class:`~abaqus.Field.DiscreteField.DiscreteField` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        location: SymbolicConstant = NODES,
        dataWidth: int = 1,
        data: Optional[DataTableArray] = None,
        description: str = "",
        orientationType: SymbolicConstant = CARTESIAN,
        partLevelOrientation: Boolean = OFF,
    ):
        """This method modifies the DiscreteField object.

        Parameters
        ----------
        location
            A SymbolicConstant or an Int specifying the location of the domain data. Possible values
            are NODES and ELEMENTS. The default value is NODES.
        dataWidth
            An Int specifying the width of the supplied data. The default value is 1.
        data
            A :py:class:`~abaqus.Field.DataTableArray.DataTableArray` object.
        description
            A String specifying the description of the field. The default value is an empty string.
        orientationType
            A SymbolicConstant specifying the type of the system being described by a discrete field
            used for an orientation. Possible values are CARTESIAN, CYLINDRICAL, and SPHERICAL. The
            default value is CARTESIAN.
        partLevelOrientation
            A Boolean specifying whether or not the orientations are described in terms of part
            level coordinates. The default value is OFF.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from .InteractionState import InteractionState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class ActuatorSensorState(InteractionState):
    """The ActuatorSensorState object stores the propagating data of an actuator sensor in a
    step. One instance of this object is created internally by the ActuatorSensor object for
    each step. The instance is also deleted internally by the ActuatorSensor object.
    The ActuatorSensorState object has no constructor, methods, or members.
    The ActuatorSensorState object is derived from the InteractionState object.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]
    """

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Optional, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .AnalyticSurface import AnalyticSurface
from .OdbAssemblyBase import OdbAssemblyBase
from .OdbDatumCsys import OdbDatumCsys
from .OdbInstance import OdbInstance
from .OdbMeshNode import OdbMeshNode
from .OdbPart import OdbPart
from .OdbRigidBody import OdbRigidBody
from .OdbSet import OdbSet
from ..UtilityAndView.abaqusConstants import Boolean, INPUT, ON, SymbolicConstant


@abaqus_class_doc
class OdbAssembly(OdbAssemblyBase):
    @abaqus_method_doc
    def DatumCsysByThreePoints(
        self,
        name: str,
        coordSysType: SymbolicConstant,
        origin: tuple,
        point1: tuple,
        point2: tuple,
    ):
        """This method creates an OdbDatumCsys object using three points. A datum coordinate system
        created with this method results in a fixed system.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreePoints

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        origin
            A sequence of Floats specifying the coordinates of the origin of the datum coordinate
            system.
        point1
            A sequence of Floats specifying the coordinates of a point on the local 1- or rr-axis.
        point2
            A sequence of Floats specifying the coordinates of a point in the 1-2 or rr-θθ plane.

        Returns
        -------
        OdbDatumCsys
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object.
        """
        self.datumCsyses[name] = datumCsys = OdbDatumCsys()
        return datumCsys

    @abaqus_method_doc
    def DatumCsysByThreeNodes(
        self,
        name: str,
        coordSysType: SymbolicConstant,
        origin: OdbMeshNode,
        point1: OdbMeshNode,
        point2: OdbMeshNode,
    ):
        """This method creates an OdbDatumCsys object using the coordinates of three OdbMeshNode
        objects. A datum coordinate system created with this method results in a system that
        follows the position of the three nodes. Results, such as those for displacement, are
        resolved into the orientation of the datum coordinate system without regard to the
        position of its origin. The last three arguments are given in the form of an OdbMeshNode
        object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreeNodes

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        origin
            An :py:class:`~abaqus.Odb.OdbMeshNode.OdbMeshNode` object specifying a node at the origin of the datum coordinate system.
        point1
            An :py:class:`~abaqus.Odb.OdbMeshNode.OdbMeshNode` object specifying a node on the local 1- or rr-axis.
        point2
            An :py:class:`~abaqus.Odb.OdbMeshNode.OdbMeshNode` object specifying a node in the 1-2 or rr-θθ plane.

        Returns
        -------
        OdbDatumCsys
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object.
        """
        self.datumCsyses[name] = datumCsys = OdbDatumCsys()
        return datumCsys

    @abaqus_method_doc
    def DatumCsysByThreeCircNodes(
        self,
        name: str,
        coordSysType: SymbolicConstant,
        node1Arc: OdbMeshNode,
        node2Arc: OdbMeshNode,
        node3Arc: OdbMeshNode,
    ):
        """This method is convenient to use where there are no nodes along the axis of a hollow
        cylinder or at the center of a hollow sphere. The three nodes that you provide as
        arguments determine a circle in space. The center of the circle is the origin of the
        datum coordinate system. The normal to the circle is parallel to the zz-axis of a
        cylindrical coordinate system or to the ϕϕ-axis of a spherical coordinate system. The
        line from the origin to the first node defines the rr-axis.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysByThreeCircNodes

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        node1Arc
            An :py:class:`~abaqus.Odb.OdbMeshNode.OdbMeshNode` object that lies on the circular arc.
        node2Arc
            An :py:class:`~abaqus.Odb.OdbMeshNode.OdbMeshNode` object that lies on the circular arc.
        node3Arc
            An :py:class:`~abaqus.Odb.OdbMeshNode.OdbMeshNode` object that lies on the circular arc.

        Returns
        -------
        OdbDatumCsys
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object.
        """
        self.datumCsyses[name] = datumCsys = OdbDatumCsys()
        return datumCsys

    @abaqus_method_doc
    def DatumCsysBy6dofNode(
        self, name: str, coordSysType: SymbolicConstant, origin: OdbMeshNode
    ):
        """A datum coordinate system created with this method results in a system that follows the
        position of a node. The node location defines the origin of the datum coordinate system.
        The rotational displacement (UR1, UR2, UR3) of the node defines the orientation of the
        coordinate system axes. Results, such as those for displacement, are resolved into the
        orientation of the datum coordinate system without regard to the position of its origin.
        The last argument is given in the form of an OdbMeshNode object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsysBy6dofNode

        Parameters
        ----------
        name
            A String specifying the repository key.
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        origin
            An :py:class:`~abaqus.Odb.OdbMeshNode.OdbMeshNode` object specifying the origin of the datum coordinate system.

        Returns
        -------
        OdbDatumCsys
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object.
        """
        self.datumCsyses[name] = datumCsys = OdbDatumCsys()
        return datumCsys

    @abaqus_method_doc
    def DatumCsys(self, name: str, datumCsys: OdbDatumCsys):
        """This method copies oneOdbDatumCsys object to a new OdbDatumCsys object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].rootAssembly.DatumCsys

        Parameters
        ----------
        name
            A String specifying the repository key.
        datumCsys
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object specifying the object to be copied.

        Returns
        -------
        OdbDatumCsys
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object.
        """
        self.datumCsyses[name] = datumCsys = OdbDatumCsys()
        return datumCsys

    @abaqus_method_doc
    def Instance(
        self, name: str, object: OdbPart, localCoordSystem: tuple = ()
    ) -> OdbInstance:
        """This method creates an OdbInstance object from an OdbPart object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].rootAssembly.Instance

        Parameters
        ----------
        name
            A String specifying the instance name.
        object
            An :py:class:`~abaqus.Odb.OdbPart.OdbPart` object.
        localCoordSystem
            A sequence of sequences of three Floats specifying the rotation and translation of the
            part instance in the global Cartesian coordinate system. The first three sequences
            specify the new local coordinate system with its center at the origin.The first sequence
            specifies a point on the 1-axis.The second sequence specifies a point on the 2-axis.The
            third sequence specifies a point on the 3-axis.The fourth sequence specifies the
            translation of the local coordinate system from the origin to its intended location.For
            example, the following sequence moves a part 10 units in the **X**-direction with no
            rotation:`localCoordSystem = ((1, 0, 0), (0, 1, 0), (0, 0, 1), (10, 0, 0))`The following
            sequence moves a part 5 units in the **X**-direction with rotation:
            `localCoordSystem = ((0, 1, 0), (1, 0, 0), (0, 0, 1), (5, 0, 0))`transforms a part
            containing the two points`Pt1= (1,0,0) Pt2= (2,0,0)` to `Pt1 = (0, 6, 0) Pt2 = (0, 7, 0)`

        Returns
        -------
        OdbInstance
            An :py:class:`~abaqus.Odb.OdbInstance.OdbInstance` object.
        """
        self.instances[name] = odbInstance = OdbInstance(name, object, localCoordSystem)
        return odbInstance

    @abaqus_method_doc
    def RigidBody(
        self,
        referenceNode: OdbSet,
        position: SymbolicConstant = INPUT,
        isothermal: Boolean = ON,
        elements: OdbSet = OdbSet("set", ()),
        tieNodes: OdbSet = OdbSet("set", ()),
        pinNodes: OdbSet = OdbSet("set", ()),
        analyticSurface: Optional[AnalyticSurface] = None, 
    ) -> OdbRigidBody:
        """This method creates a OdbRigidBody object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].rootAssembly.instances[name].RigidBody
                session.odbs[name].rootAssembly.RigidBody

        Parameters
        ----------
        referenceNode
            An :py:class:`~abaqus.Odb.OdbSet.OdbSet` object specifying the reference node set associated with the rigid body.
        position
            A SymbolicConstant specifying the specific location of the OdbRigidBody reference node
            relative to the rest of the rigid body. Possible values are INPUT and CENTER_OF_MASS.
            The default value is INPUT.
        isothermal
            A Boolean specifying specify whether the OdbRigidBody can have temperature gradients or
            be isothermal. This is used only for fully coupled thermal-stress analysis The default
            value is ON.
        elements
            An :py:class:`~abaqus.Odb.OdbSet.OdbSet` object specifying the element set whose motion is governed by the motion of
            rigid body reference node.
        tieNodes
            An :py:class:`~abaqus.Odb.OdbSet.OdbSet` object specifying the node set which have both translational and rotational
            degrees of freedom associated with the rigid body.
        pinNodes
            An :py:class:`~abaqus.Odb.OdbSet.OdbSet` object specifying the node set which have only translational degrees of
            freedom associated with the rigid body.
        analyticSurface
            An :py:class:`~abaqus.Odb.AnalyticSurface.AnalyticSurface` object specifying the analytic surface whose motion is governed by
            the motion of rigid body reference node.

        Returns
        -------
        OdbRigidBody
            An :py:class:`~abaqus.Odb.OdbRigidBody.OdbRigidBody` object.
        """
        odbRigidBody = OdbRigidBody(
            referenceNode,
            position,
            isothermal,
            elements,
            tieNodes,
            pinNodes,
            analyticSurface,
        )
        self.rigidBodies.append(odbRigidBody)
        return odbRigidBody

    @abaqus_method_doc
    def NodeSet(self, name: str, nodes: Sequence[OdbMeshNode]) -> OdbSet:
        """This method creates a node set from an array of OdbMeshNode objects (for part
        instance-level sets) or from a sequence of arrays of OdbMeshNode objects (for
        assembly-level sets).

        .. note:: 
            This function can be accessed by::

                session.odbs[name].parts[name].NodeSet
                session.odbs[name].rootAssembly.instances[name].NodeSet
                session.odbs[name].rootAssembly.NodeSet

        Parameters
        ----------
        name
            A String specifying the name of the set and the repository key.
        nodes
            A sequence of OdbMeshNode objects. For example, for a part:`nodes=part1.nodes[1:5]`For
            an assembly:`nodes=(instance1.nodes[6:7], instance2.nodes[1:5])`

        Returns
        -------
        OdbSet
            An :py:class:`~abaqus.Odb.OdbSet.OdbSet` object.
        """
        self.nodeSets[name] = odbSet = OdbSet(name, nodes)
        return odbSet



from typing import Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .CrushStressVelocityFactor import CrushStressVelocityFactor
from ....UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class CrushStress:
    """The CrushStress object specifies the crush stress of a material.


    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].crushStress
            import odbMaterial
            session.odbs[name].materials[name].crushStress

        The table data for this object are:

        - Scaling factor.
        - Relative velocity.

        The corresponding analysis keywords are:

        - CRUSH STRESS

    .. versionadded:: 2022
        The `CrushStress` class was added.
    """

    #: A sequence of sequences of Floats specifying the items described below.
    crushStressTable: Sequence[Sequence[float]]

    #: A Boolean specifying whether the data depend on temperature. The default value is OFF.
    temperatureDependency: Boolean = OFF

    # An Int specifying the number of field variable dependencies. The default value is 0.
    dependencies: int = 0

    #: A :py:class:`~abaqus.Material.Plastic.CrushStress.CrushStressVelocityFactor.CrushStressVelocityFactor` object.
    crushStressVelocityFactor: CrushStressVelocityFactor = CrushStressVelocityFactor(
        ((),)
    )

    @abaqus_method_doc
    def __init__(
        self,
        crushStressTable: Sequence[Sequence[float]],
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a CrushStress object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].CrushStress
                session.odbs[name].materials[name].CrushStress

        Parameters
        ----------
        crushStressTable
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        """
        self.crushStressTable = crushStressTable
        self.temperatureDependency = temperatureDependency
        self.dependencies = dependencies

    @abaqus_method_doc
    def setValues(
        self,
        crushStressTable: Sequence[Sequence[float]] = ((),),
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a CrushStress object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].CrushStress
                session.odbs[name].materials[name].CrushStress

        Parameters
        ----------
        crushStressTable
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        """
        self.crushStressTable = crushStressTable
        self.temperatureDependency = temperatureDependency
        self.dependencies = dependencies



from typing import Dict, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .PropertyTableData import PropertyTableData
from ..UtilityAndView.abaqusConstants import Boolean, OFF, SymbolicConstant


@abaqus_class_doc
class PropertyTable:
    """A PropertyTable is an object that is used to define the container that encapsulates the
    PropertyTableData object.
    The data of the PropertyTableData object is dependent on the contents of the
    PropertyTable object.
    After PropertyTableDatais instantiated, making changes to PropertyTable may lead to data
    corruption.

    .. note:: 
        This object can be accessed by::

            mdb.models[name].tableCollections[name].propertyTables[name]

        The corresponding analysis keywords are:

        - PROPERTY TABLE TYPE
        - PROPERTY TABLE

    .. versionadded:: 2020
        The `PropertyTable` class was added.
    """

    #: A repository of PropertyTableData. Specifies all the propertyTableData in PropertyTable
    propertyTableDatas: Dict[str, PropertyTableData] = {}

    #: A String specifying the repository key.
    name: str

    #: A string array specifying the multiple properties to build the parameter table type.
    properties: str

    #: A String array specifying multiple independent variables. The default value is an empty
    #: array.
    variables: str = ""

    @abaqus_method_doc
    def __init__(self, name: str, properties: str, variables: str = ""):
        """This method creates a PropertyTable object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].tableCollections[name].PropertyTable

        Parameters
        ----------
        name
            A String specifying the repository key.
        properties
            A string array specifying the multiple properties to build the parameter table type.
        variables
            A String array specifying multiple independent variables. The default value is an empty
            array.

        Returns
        -------
        PropertyTable
            A :py:class:`~abaqus.TableCollection.PropertyTable.PropertyTable` object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, variables: str = ""):
        """This method modifies the PropertyTable object.

        Parameters
        ----------
        variables
            A String array specifying multiple independent variables. The default value is an empty
            array.

        Returns
        -------

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def PropertyTableData(
        self,
        label: str = "",
        regularize: Optional[SymbolicConstant] = None,
        extrapolate: Optional[SymbolicConstant] = None,
        isTemp: Boolean = OFF,
        fieldNums: Optional[int] = None,
        regularizeTolerance: str = "",
        data: str = "",
    ) -> PropertyTableData:
        """This method creates a PropertyTableData object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].tableCollections[name].PropertyTable

        Parameters
        ----------
        label
            A String specifying a unique label name for the current PropertyTable object.
        regularize
            A SymbolicConstant specifying the type of regularize to the user-defined property data.
        extrapolate
            A SymbolicConstant specifying the type of extrapolation of dependent variables outside
            the specified range of the independent variables.
        isTemp
            A Boolean specifying the dependency of properties on temperature.
        fieldNums
            An Int specifying the field variables on which properties are dependent.
        regularizeTolerance
            A Double specifying the tolerance to be used to regularize the property table data.
        data
            An Array of doubles specifying the values of the properties, the variables mentioned in
            PropertyTable, and the field variables mentioned in PropertyTableData.

        Returns
        -------
        PropertyTableData
            A :py:class:`~abaqus.TableCollection.PropertyTableData.PropertyTableData` object.

        Raises
        ------
        RangeError
        """
        self.propertyTableDatas[label] = propertyTableData = PropertyTableData(
            label, regularize, extrapolate, isTemp, fieldNums, regularizeTolerance, data
        )
        return propertyTableData
